"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute = execute;
const lodash_1 = __importDefault(require("lodash"));
const constants_1 = require("../constants");
const POWER_SHELL_SCRIPT = 'powerShell';
const EXECUTE_SCRIPT_PREFIX = 'windows:';
/**
 *
 * @this {WindowsDriver}
 * @param {string} script
 * @param {ExecuteMethodArgs} [args]
 * @returns {Promise<any>}
 */
async function execute(script, args) {
    if (script === POWER_SHELL_SCRIPT) {
        this.assertFeatureEnabled(constants_1.POWER_SHELL_FEATURE);
        return await this.execPowerShell(
        /** @type {import('./powershell').ExecPowerShellOptions} */ (preprocessExecuteMethodArgs(args)));
    }
    this.log.info(`Executing extension command '${script}'`);
    const formattedScript = lodash_1.default.isString(script)
        ? script.trim().replace(/^windows:\s*/, `${EXECUTE_SCRIPT_PREFIX} `)
        : String(script);
    const preprocessedArgs = preprocessExecuteMethodArgs(args);
    return await this.executeMethod(formattedScript, [preprocessedArgs]);
}
/**
 * Massages the arguments going into an execute method.
 *
 * @param {ExecuteMethodArgs} [args]
 * @returns {StringRecord}
 */
function preprocessExecuteMethodArgs(args) {
    return /** @type {StringRecord} */ ((lodash_1.default.isArray(args) ? lodash_1.default.first(args) : args) ?? {});
}
/**
 * @typedef {import('../driver').WindowsDriver} WindowsDriver
 * @typedef {import('@appium/types').StringRecord} StringRecord
 * @typedef {readonly any[] | readonly [StringRecord] | Readonly<StringRecord>} ExecuteMethodArgs
 */
//# sourceMappingURL=execute.js.map