/**
 * @typedef {Object} ExecPowerShellOptions
 * @property {string} [script] A valid Power Shell script to execute
 * @property {string} [command] A valid Power Shell command to execute
 */
/**
 * Executes the given Power Shell command or a whole script based on the
 * given options. Either of these options must be provided. If both are provided
 * then the `command` one gets the priority.
 * Note that Power Shell command cannot contain line breaks. Consider making it
 * to a script in such case.
 * Note that by default Power Shell blocks scripts execution, so the script must
 * temporarily switch user execution policy if necessary and restore it afterwards.
 * This makes scripts slightly less performant, as single commands.
 *
 * @this {WindowsDriver}
 * @param {ExecPowerShellOptions} opts
 * @returns {Promise<string>} The actual stdout of the given command/script
 * @throws {Error} If the exit code of the given command/script is not zero.
 * The actual stderr output is set to the error message value.
 */
export function execPowerShell(this: import("../driver").WindowsDriver, opts: ExecPowerShellOptions): Promise<string>;
export type ExecPowerShellOptions = {
    /**
     * A valid Power Shell script to execute
     */
    script?: string | undefined;
    /**
     * A valid Power Shell command to execute
     */
    command?: string | undefined;
};
export type WindowsDriver = import("../driver").WindowsDriver;
//# sourceMappingURL=powershell.d.ts.map