/**
 * @implements {ExternalDriver<WindowsDriverConstraints, string>}
 * @extends {BaseDriver<WindowsDriverConstraints>}
 */
export class WindowsDriver extends BaseDriver<{
    readonly platformName: {
        readonly presence: true;
        readonly isString: true;
        readonly inclusionCaseInsensitive: readonly ["Windows"];
    };
    readonly browserName: {
        readonly isString: true;
    };
    readonly platformVersion: {
        readonly isString: true;
    };
    readonly app: {
        readonly isString: true;
    };
    readonly appArguments: {
        readonly isString: true;
    };
    readonly appTopLevelWindow: {
        readonly isString: true;
    };
    readonly appWorkingDir: {
        readonly isString: true;
    };
    readonly createSessionTimeout: {
        readonly isNumber: true;
    };
    readonly 'ms:waitForAppLaunch': {
        readonly isNumber: true;
    };
    readonly 'ms:forcequit': {
        readonly isBoolean: true;
    };
    readonly 'ms:experimental-webdriver': {
        readonly isBoolean: true;
    };
    readonly systemPort: {
        readonly isNumber: true;
    };
    readonly prerun: {
        readonly isObject: true;
    };
    readonly postrun: {
        readonly isObject: true;
    };
    readonly wadUrl: {
        readonly isString: true;
    };
}, import("@appium/types").StringRecord, import("@appium/types").StringRecord, import("@appium/types").DefaultCreateSessionResult<{
    readonly platformName: {
        readonly presence: true;
        readonly isString: true;
        readonly inclusionCaseInsensitive: readonly ["Windows"];
    };
    readonly browserName: {
        readonly isString: true;
    };
    readonly platformVersion: {
        readonly isString: true;
    };
    readonly app: {
        readonly isString: true;
    };
    readonly appArguments: {
        readonly isString: true;
    };
    readonly appTopLevelWindow: {
        readonly isString: true;
    };
    readonly appWorkingDir: {
        readonly isString: true;
    };
    readonly createSessionTimeout: {
        readonly isNumber: true;
    };
    readonly 'ms:waitForAppLaunch': {
        readonly isNumber: true;
    };
    readonly 'ms:forcequit': {
        readonly isBoolean: true;
    };
    readonly 'ms:experimental-webdriver': {
        readonly isBoolean: true;
    };
    readonly systemPort: {
        readonly isNumber: true;
    };
    readonly prerun: {
        readonly isObject: true;
    };
    readonly postrun: {
        readonly isObject: true;
    };
    readonly wadUrl: {
        readonly isString: true;
    };
}>, void, import("@appium/types").StringRecord> implements ExternalDriver<WindowsDriverConstraints, string> {
    static newMethodMap: {
        readonly '/session/:sessionId/appium/start_recording_screen': {
            readonly POST: {
                readonly command: "startRecordingScreen";
                readonly payloadParams: {
                    readonly optional: readonly ["options"];
                };
            };
        };
        readonly '/session/:sessionId/appium/stop_recording_screen': {
            readonly POST: {
                readonly command: "stopRecordingScreen";
                readonly payloadParams: {
                    readonly optional: readonly ["options"];
                };
            };
        };
        readonly '/session/:sessionId/appium/device/push_file': {
            readonly POST: {
                readonly command: "pushFile";
                readonly payloadParams: {
                    readonly required: readonly ["path", "data"];
                };
            };
        };
        readonly '/session/:sessionId/appium/device/pull_file': {
            readonly POST: {
                readonly command: "pullFile";
                readonly payloadParams: {
                    readonly required: readonly ["path"];
                };
            };
        };
        readonly '/session/:sessionId/appium/device/pull_folder': {
            readonly POST: {
                readonly command: "pullFolder";
                readonly payloadParams: {
                    readonly required: readonly ["path"];
                };
            };
        };
        readonly '/session/:sessionId/window/:windowhandle/size': {
            readonly GET: {
                readonly command: "getWindowSize";
            };
            readonly POST: {};
        };
        readonly '/session/:sessionId/window/:windowhandle/position': {
            readonly POST: {};
            readonly GET: {};
        };
        readonly '/session/:sessionId/window_handle': {
            readonly GET: {
                readonly command: "getWindowHandle";
            };
        };
        readonly '/session/:sessionId/window_handles': {
            readonly GET: {
                readonly command: "getWindowHandles";
            };
        };
        readonly '/session/:sessionId/appium/app/launch': {
            readonly POST: {
                readonly command: "launchApp";
            };
        };
        readonly '/session/:sessionId/appium/app/close': {
            readonly POST: {
                readonly command: "closeApp";
            };
        };
        readonly '/session/:sessionId/click': {
            readonly POST: {
                readonly command: "clickCurrent";
                readonly payloadParams: {
                    readonly optional: readonly ["button"];
                };
            };
        };
        readonly '/session/:sessionId/buttondown': {
            readonly POST: {
                readonly command: "buttonDown";
                readonly payloadParams: {
                    readonly optional: readonly ["button"];
                };
            };
        };
        readonly '/session/:sessionId/buttonup': {
            readonly POST: {
                readonly command: "buttonUp";
                readonly payloadParams: {
                    readonly optional: readonly ["button"];
                };
            };
        };
        readonly '/session/:sessionId/doubleclick': {
            readonly POST: {
                readonly command: "doubleClick";
            };
        };
        readonly '/session/:sessionId/touch/click': {
            readonly POST: {
                readonly command: "click";
                readonly payloadParams: {
                    readonly required: readonly ["element"];
                };
            };
        };
        readonly '/session/:sessionId/touch/down': {
            readonly POST: {
                readonly command: "touchDown";
                readonly payloadParams: {
                    readonly required: readonly ["x", "y"];
                };
            };
        };
        readonly '/session/:sessionId/touch/up': {
            readonly POST: {
                readonly command: "touchUp";
                readonly payloadParams: {
                    readonly required: readonly ["x", "y"];
                };
            };
        };
        readonly '/session/:sessionId/touch/move': {
            readonly POST: {
                readonly command: "touchMove";
                readonly payloadParams: {
                    readonly required: readonly ["x", "y"];
                };
            };
        };
        readonly '/session/:sessionId/touch/longclick': {
            readonly POST: {
                readonly command: "touchLongClick";
                readonly payloadParams: {
                    readonly required: readonly ["elements"];
                };
            };
        };
        readonly '/session/:sessionId/touch/flick': {
            readonly POST: {
                readonly command: "flick";
                readonly payloadParams: {
                    readonly optional: readonly ["element", "xspeed", "yspeed", "xoffset", "yoffset", "speed"];
                };
            };
        };
        readonly '/session/:sessionId/touch/perform': {
            readonly POST: {
                readonly command: "performTouch";
                readonly payloadParams: {
                    readonly wrap: "actions";
                    readonly required: readonly ["actions"];
                };
            };
        };
        readonly '/session/:sessionId/touch/multi/perform': {
            readonly POST: {
                readonly command: "performMultiAction";
                readonly payloadParams: {
                    readonly required: readonly ["actions"];
                    readonly optional: readonly ["elementId"];
                };
            };
        };
        readonly '/session/:sessionId/keys': {
            readonly POST: {
                readonly command: "keys";
                readonly payloadParams: {
                    readonly required: readonly ["value"];
                };
            };
        };
        readonly '/session/:sessionId/element/:elementId/location': {
            readonly GET: {
                readonly command: "getLocation";
            };
        };
        readonly '/session/:sessionId/element/:elementId/location_in_view': {
            readonly GET: {
                readonly command: "getLocationInView";
            };
        };
        readonly '/session/:sessionId/element/:elementId/size': {
            readonly GET: {
                readonly command: "getSize";
            };
        };
        readonly '/session/:sessionId/element/:elementId/equals/:otherId': {
            readonly GET: {
                readonly command: "equalsElement";
            };
        };
    };
    static executeMethodMap: {
        readonly 'windows: startRecordingScreen': {
            readonly command: "windowsStartRecordingScreen";
            readonly params: {
                readonly optional: readonly ["timeLimit", "videoFilter", "fps", "preset", "captureCursor", "captureClicks", "audioInput", "forceRestart"];
            };
        };
        readonly 'windows: stopRecordingScreen': {
            readonly command: "windowsStopRecordingScreen";
            readonly params: {
                readonly optional: readonly ["remotePath", "user", "pass", "method", "headers", "fileFieldName", "formFields"];
            };
        };
        readonly 'windows: launchApp': {
            readonly command: "windowsLaunchApp";
        };
        readonly 'windows: closeApp': {
            readonly command: "windowsCloseApp";
        };
        readonly 'windows: deleteFolder': {
            readonly command: "windowsDeleteFolder";
            readonly params: {
                readonly required: readonly ["remotePath"];
            };
        };
        readonly 'windows: deleteFile': {
            readonly command: "windowsDeleteFile";
            readonly params: {
                readonly required: readonly ["remotePath"];
            };
        };
        readonly 'windows: click': {
            readonly command: "windowsClick";
            readonly params: {
                readonly optional: readonly ["elementId", "x", "y", "button", "modifierKeys", "durationMs", "times", "interClickDelayMs"];
            };
        };
        readonly 'windows: scroll': {
            readonly command: "windowsScroll";
            readonly params: {
                readonly optional: readonly ["elementId", "x", "y", "deltaX", "deltaY", "modifierKeys"];
            };
        };
        readonly 'windows: clickAndDrag': {
            readonly command: "windowsClickAndDrag";
            readonly params: {
                readonly optional: readonly ["startElementId", "startX", "startY", "endElementId", "endX", "endY", "modifierKeys", "durationMs"];
            };
        };
        readonly 'windows: hover': {
            readonly command: "windowsHover";
            readonly params: {
                readonly optional: readonly ["startElementId", "startX", "startY", "endElementId", "endX", "endY", "modifierKeys", "durationMs"];
            };
        };
        readonly 'windows: keys': {
            readonly command: "windowsKeys";
            readonly params: {
                readonly required: readonly ["actions"];
            };
        };
        readonly 'windows: setClipboard': {
            readonly command: "windowsSetClipboard";
            readonly params: {
                readonly required: readonly ["b64Content"];
                readonly optional: readonly ["contentType"];
            };
        };
        readonly 'windows: getClipboard': {
            readonly command: "windowsGetClipboard";
            readonly params: {
                readonly optional: readonly ["contentType"];
            };
        };
    };
    constructor(opts?: {}, shouldValidateCaps?: boolean);
    /** @type {boolean} */
    isProxyActive: boolean;
    /** @type {import('@appium/types').RouteMatcher[]} */
    jwpProxyAvoid: import("@appium/types").RouteMatcher[];
    /** @type {WinAppDriver} */
    winAppDriver: WinAppDriver;
    /** @type {import('./commands/record-screen').ScreenRecorder | null} */
    _screenRecorder: import("./commands/record-screen").ScreenRecorder | null;
    resetState(): void;
    createSession(...args: any[]): Promise<(string | import("@appium/types").DriverCaps<{
        readonly platformName: {
            readonly presence: true;
            readonly isString: true;
            readonly inclusionCaseInsensitive: readonly ["Windows"];
        };
        readonly browserName: {
            readonly isString: true;
        };
        readonly platformVersion: {
            readonly isString: true;
        };
        readonly app: {
            readonly isString: true;
        };
        readonly appArguments: {
            readonly isString: true;
        };
        readonly appTopLevelWindow: {
            readonly isString: true;
        };
        readonly appWorkingDir: {
            readonly isString: true;
        };
        readonly createSessionTimeout: {
            readonly isNumber: true;
        };
        readonly 'ms:waitForAppLaunch': {
            readonly isNumber: true;
        };
        readonly 'ms:forcequit': {
            readonly isBoolean: true;
        };
        readonly 'ms:experimental-webdriver': {
            readonly isBoolean: true;
        };
        readonly systemPort: {
            readonly isNumber: true;
        };
        readonly prerun: {
            readonly isObject: true;
        };
        readonly postrun: {
            readonly isObject: true;
        };
        readonly wadUrl: {
            readonly isString: true;
        };
    }>)[]>;
    startWinAppDriverSession(): Promise<void>;
    proxyReqRes: any;
    deleteSession(): Promise<void>;
    proxyActive(sessionId: any): boolean;
    canProxy(): boolean;
    getProxyAvoidList(sessionId: any): import("@appium/types").RouteMatcher[];
    proxyCommand(url: any, method: any, body: any): Promise<any>;
    windowsLaunchApp: typeof appManagementCommands.windowsLaunchApp;
    windowsCloseApp: typeof appManagementCommands.windowsCloseApp;
    windowsSetClipboard: typeof clipboardCommands.windowsSetClipboard;
    windowsGetClipboard: typeof clipboardCommands.windowsGetClipboard;
    execute: typeof executeCommands.execute;
    pushFile: typeof fileCommands.pushFile;
    pullFile: typeof fileCommands.pullFile;
    pullFolder: typeof fileCommands.pullFolder;
    windowsDeleteFile: typeof fileCommands.windowsDeleteFile;
    windowsDeleteFolder: typeof fileCommands.windowsDeleteFolder;
    findElOrEls: typeof findCommands.findElOrEls;
    getWindowSize: typeof generalCommands.getWindowSize;
    getWindowRect: typeof generalCommands.getWindowRect;
    setWindowRect: typeof generalCommands.setWindowRect;
    getScreenshot: typeof generalCommands.getScreenshot;
    getElementRect: typeof generalCommands.getElementRect;
    windowsClick: typeof gestureCommands.windowsClick;
    windowsScroll: typeof gestureCommands.windowsScroll;
    windowsClickAndDrag: typeof gestureCommands.windowsClickAndDrag;
    windowsHover: typeof gestureCommands.windowsHover;
    windowsKeys: typeof gestureCommands.windowsKeys;
    execPowerShell: typeof powershellCommands.execPowerShell;
    windowsStartRecordingScreen: typeof recordScreenCommands.windowsStartRecordingScreen;
    windowsStopRecordingScreen: typeof recordScreenCommands.windowsStopRecordingScreen;
    startRecordingScreen: typeof recordScreenCommands.startRecordingScreen;
    stopRecordingScreen: typeof recordScreenCommands.stopRecordingScreen;
    performActions: typeof touchCommands.performActions;
    getContexts: typeof contextCommands.getContexts;
    getCurrentContext: typeof contextCommands.getCurrentContext;
    setContext: typeof contextCommands.setContext;
    supportedLogTypes: import("@appium/types").LogDefRecord;
}
export default WindowsDriver;
export type WindowsDriverConstraints = typeof desiredCapConstraints;
export type WindowsDriverOpts = import("@appium/types").DriverOpts<WindowsDriverConstraints>;
export type ExternalDriver<C extends import("@appium/types").Constraints, Ctx = string> = import("@appium/types").ExternalDriver<C, Ctx>;
import { BaseDriver } from 'appium/driver';
import { WinAppDriver } from './winappdriver';
import * as appManagementCommands from './commands/app-management';
import * as clipboardCommands from './commands/clipboard';
import * as executeCommands from './commands/execute';
import * as fileCommands from './commands/file-movement';
import * as findCommands from './commands/find';
import * as generalCommands from './commands/general';
import * as gestureCommands from './commands/gestures';
import * as powershellCommands from './commands/powershell';
import * as recordScreenCommands from './commands/record-screen';
import * as touchCommands from './commands/touch';
import * as contextCommands from './commands/context';
import { desiredCapConstraints } from './desired-caps';
//# sourceMappingURL=driver.d.ts.map