export type RegEntry = {
    /**
     * Full path to the registry branch, for example
     * HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\DirectDrawEx
     */
    root: string;
    /**
     * The registry key name
     */
    key: string;
    /**
     * One of possible registry value types, for example
     * REG_DWORD or REG_SZ
     */
    type: string;
    /**
     * The actual value. Could be empty
     */
    value: string;
};
/**
 * @typedef {Object} RegEntry
 * @property {string} root Full path to the registry branch, for example
 * HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\DirectDrawEx
 * @property {string} key The registry key name
 * @property {string} type One of possible registry value types, for example
 * REG_DWORD or REG_SZ
 * @property {string} value The actual value. Could be empty
 */
/**
 * Lists registry tree (e.g. recursively) under the given root node.
 * The lookup is done under the same registry branch that the current process
 * system architecture.
 *
 * @param {string} root The registry key name, which consists of two parts:
 * - The root key: HKLM | HKCU | HKCR | HKU | HKCC
 * - The subkey under the selected root key, for example \Software\Microsoft
 * @returns {Promise<RegEntry[]>} List of matched RegEntry instances or an empty list
 * if either no entries were found under the given root or the root does not exist.
 */
export function queryRegistry(root: string): Promise<RegEntry[]>;
export function parseRegQueryOutput(output: any): any[];
//# sourceMappingURL=registry.d.ts.map