"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.shellExec = shellExec;
exports.downloadToFile = downloadToFile;
const support_1 = require("appium/support");
const node_util_1 = require("node:util");
const node_child_process_1 = require("node:child_process");
const bluebird_1 = __importDefault(require("bluebird"));
const execAsync = (0, node_util_1.promisify)(node_child_process_1.exec);
/**
 * This API triggers UAC when necessary
 * unlike the 'spawn' call used by teen_process's exec.
 * See https://github.com/nodejs/node-v0.x-archive/issues/6797
 *
 * @param {string} cmd
 * @param {string[]} args
 * @param {import('node:child_process').ExecOptions & {timeoutMs?: number}} opts
 * @returns {Promise<{stdout: string; stderr: string;}>}
 * @throws {import('node:child_process').ExecException}
 */
async function shellExec(cmd, args = [], opts = {}) {
    const { timeoutMs = 60 * 1000 * 5 } = opts;
    const fullCmd = support_1.util.quote([cmd, ...args]);
    return await bluebird_1.default.resolve(execAsync(fullCmd, opts))
        .timeout(timeoutMs, `The command '${fullCmd}' timed out after ${timeoutMs}ms`);
}
/**
 *
 * @param {string} srcUrl
 * @param {string} dstPath
 * @returns {Promise<void>}
 */
async function downloadToFile(srcUrl, dstPath) {
    await support_1.net.downloadFile(srcUrl, dstPath);
}
//# sourceMappingURL=utils.js.map